from __future__ import absolute_import
import numpy.fft as ffto
from .numpy_wrapper import wrap_namespace
from . import numpy_wrapper as anp

wrap_namespace(ffto.__dict__, globals())

fft.defgrad(    lambda ans, x, n=None, axis=-1       : lambda g : fft(g, n, axis))
ifft.defgrad(   lambda ans, x, n=None, axis=-1       : lambda g : ifft(g, n, axis))
fft2.defgrad(   lambda ans, x, s=None, axes=(-2, -1) : lambda g : fft2(g, s, axes))
ifft2.defgrad(  lambda ans, x, s=None, axes=(-2, -1) : lambda g : ifft2(g, s, axes))
fftn.defgrad(   lambda ans, x, s=None, axes=None     : lambda g : fftn(g, s, axes))
ifftn.defgrad(  lambda ans, x, s=None, axes=None     : lambda g : ifftn(g, s, axes))

#TODO: Support n and s arguments for all the above methods.

fftshift.defgrad( lambda ans, x, axes=None : lambda g : anp.conj(ifftshift(anp.conj(g), axes)))
ifftshift.defgrad(lambda ans, x, axes=None : lambda g : anp.conj(fftshift(anp.conj(g), axes)))



from use_gpu_numpy import use_gpu_numpy
import numpy_wrapper
import numpy_grads
import numpy_extra
import complex_array_node

if use_gpu_numpy():
    import gpu_array_node

from numpy_wrapper import *
from . import linalg
from . import fft
from . import random



from .core import grad, primitive
import container_types
from convenience_wrappers import (elementwise_grad, jacobian, value_and_grad,
                                  grad_and_aux, hessian_vector_product, hessian)
